@@ -1,7 +1,12 @@
+pub mod attribute_mapping;
+pub mod cardinality;
 pub mod character_encoding;
+pub mod conditional;
+pub mod conformance;
 pub mod credential_layout;
 pub mod entry;
 pub mod entry_code;
+pub mod entry_code_mapping;
 pub mod form_layout;
 pub mod format;
 pub mod information;
@@ -9,10 +14,15 @@ pub mod label;
 pub mod meta;
 pub mod unit;
 
+pub use self::attribute_mapping::AttributeMappingOverlay as AttributeMapping;
+pub use self::cardinality::CardinalityOverlay as Cardinality;
 pub use self::character_encoding::CharacterEncodingOverlay as CharacterEncoding;
+pub use self::conditional::ConditionalOverlay as Conditional;
+pub use self::conformance::ConformanceOverlay as Conformance;
 pub use self::credential_layout::CredentialLayoutOverlay as CredentialLayout;
 pub use self::entry::EntryOverlay as Entry;
 pub use self::entry_code::EntryCodeOverlay as EntryCode;
+pub use self::entry_code_mapping::EntryCodeMappingOverlay as EntryCodeMapping;
 pub use self::form_layout::FormLayoutOverlay as FormLayout;
 pub use self::format::FormatOverlay as Format;
 pub use self::information::InformationOverlay as Information;
@@ -29,6 +39,9 @@ pub trait Overlay: erased_serde::Serialize {
     fn language(&self) -> Option<&Language> {
         None
     }
+    fn metric_system(&self) -> Option<&String> {
+        None
+    }
     fn attributes(&self) -> Vec<&String>;
 
     fn add(&mut self, attribute: &Attribute);
